@@ -279,21 +279,21 @@ static std::expected<int64_t, std::string> configStringToInt(const std::string&
     return 0;
 }
 
-CParseResult CConfig::configSetValueSafe(const std::string& command, const std::string& value) {
+// found, result
+std::pair<bool, CParseResult> CConfig::configSetValueSafe(const std::string& command, const std::string& value) {
     CParseResult result;
 
     std::string  valueName;
-    std::string  catPrefix;
     for (auto& c : impl->categories) {
         valueName += c + ':';
-        catPrefix += c + ':';
     }
 
     valueName += command;
 
-    const auto VALUEONLYNAME = command.starts_with(catPrefix) ? command.substr(catPrefix.length()) : command;
+    // TODO: all this sucks xD
+
+    SSpecialCategory* overrideSpecialCat = nullptr;
 
-    // FIXME: this will bug with nested.
     if (valueName.contains('[') && valueName.contains(']')) {
         const auto L = valueName.find_first_of('[');
         const auto R = valueName.find_last_of(']');
@@ -304,12 +304,27 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
 
             valueName = valueName.substr(0, L) + valueName.substr(R + 1);
 
-            // if it doesn't exist, make it
             for (auto& sc : impl->specialCategoryDescriptors) {
-                if (sc->key.empty() || !valueName.starts_with(sc->name))
+                if (sc->key.empty() || !valueName.starts_with(sc->name + ":"))
                     continue;
 
-                // bingo
+                bool keyExists = false;
+                for (const auto& specialCat : impl->specialCategories) {
+                    if (specialCat->key != sc->key || specialCat->name != sc->name)
+                        continue;
+
+                    if (CATKEY != std::string_view{std::any_cast<const char*>(specialCat->values[sc->key].getValue())})
+                        continue;
+
+                    // existing special
+                    keyExists          = true;
+                    overrideSpecialCat = specialCat.get();
+                }
+
+                if (keyExists)
+                    break;
+
+                // if it doesn't exist, make it
                 const auto PCAT  = impl->specialCategories.emplace_back(std::make_unique<SSpecialCategory>()).get();
                 PCAT->descriptor = sc.get();
                 PCAT->name       = sc->name;
@@ -319,6 +334,8 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
                 applyDefaultsToCat(*PCAT);
 
                 PCAT->values[sc->key].setFrom(CATKEY);
+                overrideSpecialCat = PCAT;
+                break;
             }
         }
     }
@@ -328,85 +345,95 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
         // it might be in a special category
         bool found = false;
 
-        if (impl->currentSpecialCategory && valueName.starts_with(impl->currentSpecialCategory->name)) {
-            VALUEIT = impl->currentSpecialCategory->values.find(valueName.substr(impl->currentSpecialCategory->name.length() + 1));
+        if (overrideSpecialCat) {
+            VALUEIT = overrideSpecialCat->values.find(valueName.substr(overrideSpecialCat->name.length() + 1));
 
-            if (VALUEIT != impl->currentSpecialCategory->values.end())
+            if (VALUEIT != overrideSpecialCat->values.end())
                 found = true;
-        }
+        } else {
+            if (impl->currentSpecialCategory && valueName.starts_with(impl->currentSpecialCategory->name)) {
+                VALUEIT = impl->currentSpecialCategory->values.find(valueName.substr(impl->currentSpecialCategory->name.length() + 1));
 
-        if (!found) {
-            for (auto& sc : impl->specialCategories) {
-                if (!valueName.starts_with(sc->name))
-                    continue;
-
-                if (!sc->isStatic && std::string{std::any_cast<const char*>(sc->values[sc->key].getValue())} != impl->currentSpecialKey)
-                    continue;
-
-                VALUEIT                      = sc->values.find(valueName.substr(sc->name.length() + 1));
-                impl->currentSpecialCategory = sc.get();
-
-                if (VALUEIT != sc->values.end())
+                if (VALUEIT != impl->currentSpecialCategory->values.end())
                     found = true;
-                else if (sc->descriptor->dontErrorOnMissing)
-                    return result; // will return a success, cuz we want to ignore missing
-
-                break;
             }
-        }
 
-        if (!found) {
-            // could be a dynamic category that doesnt exist yet
-            for (auto& sc : impl->specialCategoryDescriptors) {
-                if (sc->key.empty() || !valueName.starts_with(sc->name))
-                    continue;
+            if (!found) {
+                for (auto& sc : impl->specialCategories) {
+                    if (!valueName.starts_with(sc->name))
+                        continue;
+
+                    if (!sc->isStatic && std::string{std::any_cast<const char*>(sc->values[sc->key].getValue())} != impl->currentSpecialKey)
+                        continue;
+
+                    VALUEIT                      = sc->values.find(valueName.substr(sc->name.length() + 1));
+                    impl->currentSpecialCategory = sc.get();
+
+                    if (VALUEIT != sc->values.end())
+                        found = true;
+                    else if (sc->descriptor->dontErrorOnMissing)
+                        return {true, result}; // will return a success, cuz we want to ignore missing
 
-                // category does exist, check if value exists
-                if (!sc->defaultValues.contains(VALUEONLYNAME) && VALUEONLYNAME != sc->key)
                     break;
-
-                // bingo
-                const auto PCAT  = impl->specialCategories.emplace_back(std::make_unique<SSpecialCategory>()).get();
-                PCAT->descriptor = sc.get();
-                PCAT->name       = sc->name;
-                PCAT->key        = sc->key;
-                addSpecialConfigValue(sc->name.c_str(), sc->key.c_str(), CConfigValue("0"));
-
-                applyDefaultsToCat(*PCAT);
-
-                VALUEIT                      = PCAT->values.find(valueName.substr(sc->name.length() + 1));
-                impl->currentSpecialCategory = PCAT;
-
-                if (VALUEIT != PCAT->values.end())
-                    found = true;
-
-                if (sc->anonymous) {
-                    // find suitable key
-                    size_t biggest = 0;
-                    for (auto& catt : impl->specialCategories) {
-                        biggest = std::max(catt->anonymousID, biggest);
-                    }
-
-                    biggest++;
-
-                    PCAT->values[ANONYMOUS_KEY].setFrom(std::to_string(biggest));
-                    impl->currentSpecialKey = std::to_string(biggest);
-                    PCAT->anonymousID       = biggest;
-                } else {
-                    if (VALUEIT == PCAT->values.end() || VALUEIT->first != sc->key) {
-                        result.setError(std::format("special category's first value must be the key. Key for <{}> is <{}>", PCAT->name, PCAT->key));
-                        return result;
-                    }
-                    impl->currentSpecialKey = value;
                 }
+            }
 
-                break;
+            if (!found) {
+                // could be a dynamic category that doesnt exist yet
+                for (auto& sc : impl->specialCategoryDescriptors) {
+                    if (sc->key.empty() || !valueName.starts_with(sc->name + ":"))
+                        continue;
+
+                    // found value root to be a special category, get the trunk
+                    const auto VALUETRUNK = valueName.substr(sc->name.length() + 1);
+
+                    // check if trunk is a value within the special category
+                    if (!sc->defaultValues.contains(VALUETRUNK) && VALUETRUNK != sc->key)
+                        break;
+
+                    // bingo
+                    const auto PCAT  = impl->specialCategories.emplace_back(std::make_unique<SSpecialCategory>()).get();
+                    PCAT->descriptor = sc.get();
+                    PCAT->name       = sc->name;
+                    PCAT->key        = sc->key;
+                    addSpecialConfigValue(sc->name.c_str(), sc->key.c_str(), CConfigValue("0"));
+
+                    applyDefaultsToCat(*PCAT);
+
+                    VALUEIT                      = PCAT->values.find(valueName.substr(sc->name.length() + 1));
+                    impl->currentSpecialCategory = PCAT;
+
+                    if (VALUEIT != PCAT->values.end())
+                        found = true;
+
+                    if (sc->anonymous) {
+                        // find suitable key
+                        size_t biggest = 0;
+                        for (auto& catt : impl->specialCategories) {
+                            biggest = std::max(catt->anonymousID, biggest);
+                        }
+
+                        biggest++;
+
+                        PCAT->values[ANONYMOUS_KEY].setFrom(std::to_string(biggest));
+                        impl->currentSpecialKey = std::to_string(biggest);
+                        PCAT->anonymousID       = biggest;
+                    } else {
+                        if (VALUEIT == PCAT->values.end() || VALUEIT->first != sc->key) {
+                            result.setError(std::format("special category's first value must be the key. Key for <{}> is <{}>", PCAT->name, PCAT->key));
+                            return {true, result};
+                        }
+                        impl->currentSpecialKey = value;
+                    }
+
+                    break;
+                }
             }
         }
 
         if (!found) {
             result.setError(std::format("config option <{}> does not exist.", valueName));
-            return result;
+            return {false, result};
         }
     }
 
@@ -416,7 +443,7 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
             const auto INT = configStringToInt(value);
             if (!INT.has_value()) {
                 result.setError(INT.error());
-                return result;
+                return {true, result};
             }
 
             VALUEIT->second.setFrom(INT.value());
@@ -428,7 +455,7 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
                 VALUEIT->second.setFrom(std::stof(value));
             } catch (std::exception& e) {
                 result.setError(std::format("failed parsing a float: {}", e.what()));
-                return result;
+                return {true, result};
             }
             break;
         }
@@ -446,7 +473,7 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
                 VALUEIT->second.setFrom(SVector2D{.x = std::stof(LHS), .y = std::stof(RHS)});
             } catch (std::exception& e) {
                 result.setError(std::format("failed parsing a vec2: {}", e.what()));
-                return result;
+                return {true, result};
             }
             break;
         }
@@ -461,19 +488,19 @@ CParseResult CConfig::configSetValueSafe(const std::string& command, const std::
 
             if (RESULT.error) {
                 result.setError(RESULT.getError());
-                return result;
+                return {true, result};
             }
             break;
         }
         default: {
             result.setError("internal error: invalid value found (no type?)");
-            return result;
+            return {true, result};
         }
     }
 
     VALUEIT->second.m_bSetByUser = true;
 
-    return result;
+    return {true, result};
 }
 
 CParseResult CConfig::parseVariable(const std::string& lhs, const std::string& rhs, bool dynamic) {
@@ -536,9 +563,9 @@ std::optional<std::string> CConfigImpl::parseComment(const std::string& comment)
         }
 
         if (args[i] == "endif") {
-            if (!currentFlags.inAnIfBlock)
+            if (currentFlags.ifDatas.empty())
                 return "stray endif";
-            currentFlags.inAnIfBlock = false;
+            currentFlags.ifDatas.pop_back();
             break;
         }
 
@@ -549,20 +576,19 @@ std::optional<std::string> CConfigImpl::parseComment(const std::string& comment)
     }
 
     if (!ifBlockVariable.empty()) {
-        if (currentFlags.inAnIfBlock)
-            return "nested if statements are not allowed";
-
         if (ifBlockVariable.starts_with("!")) {
-            negated = true;
+            negated         = true;
             ifBlockVariable = ifBlockVariable.substr(1);
         }
 
-        currentFlags.inAnIfBlock = true;
+        CConfigImpl::SIfBlockData newIfData;
 
         if (const auto VAR = getVariable(ifBlockVariable); VAR)
-            currentFlags.ifBlockFailed = negated ? VAR->truthy() : !VAR->truthy();
+            newIfData.failed = negated ? VAR->truthy() : !VAR->truthy();
         else
-            currentFlags.ifBlockFailed = !negated;
+            newIfData.failed = !negated;
+
+        currentFlags.ifDatas.emplace_back(newIfData);
     }
 
     return std::nullopt;
@@ -632,7 +658,7 @@ CParseResult CConfig::parseLine(std::string line, bool dynamic) {
         return result;
     }
 
-    if (impl->currentFlags.inAnIfBlock && impl->currentFlags.ifBlockFailed)
+    if (!impl->currentFlags.ifDatas.empty() && impl->currentFlags.ifDatas.back().failed)
         return result;
 
     size_t lastHashPos = 0;
@@ -780,45 +806,51 @@ CParseResult CConfig::parseLine(std::string line, bool dynamic) {
 
         bool found = false;
 
-        for (auto& h : impl->handlers) {
-            // we want to handle potentially nested keywords and ensure
-            // we only call the handler if they are scoped correctly,
-            // unless the keyword is not scoped itself
-
-            const bool UNSCOPED    = !h.name.contains(":");
-            const auto HANDLERNAME = !h.name.empty() && h.name.at(0) == ':' ? h.name.substr(1) : h.name;
-
-            if (!h.options.allowFlags && !UNSCOPED) {
-                size_t colon = 0;
-                size_t idx   = 0;
-                size_t depth = 0;
-
-                while ((colon = HANDLERNAME.find(':', idx)) != std::string::npos && impl->categories.size() > depth) {
-                    auto actual = HANDLERNAME.substr(idx, colon - idx);
-
-                    if (actual != impl->categories[depth])
-                        break;
-
-                    idx = colon + 1;
-                    ++depth;
-                }
-
-                if (depth != impl->categories.size() || HANDLERNAME.substr(idx) != LHS)
-                    continue;
-            }
-
-            if (UNSCOPED && HANDLERNAME != LHS && !h.options.allowFlags)
-                continue;
-
-            if (h.options.allowFlags && (!LHS.starts_with(HANDLERNAME) || LHS.contains(':') /* avoid cases where a category is called the same as a handler */))
-                continue;
-
-            ret   = h.func(LHS.c_str(), RHS.c_str());
-            found = true;
+        if (!impl->configOptions.verifyOnly) {
+            auto [f, rv]    = configSetValueSafe(LHS, RHS);
+            found           = f;
+            ret             = std::move(rv);
+            ret.errorString = ret.errorStdString.c_str();
         }
 
-        if (!found && !impl->configOptions.verifyOnly)
-            ret = configSetValueSafe(LHS, RHS);
+        if (!found) {
+            for (auto& h : impl->handlers) {
+                // we want to handle potentially nested keywords and ensure
+                // we only call the handler if they are scoped correctly,
+                // unless the keyword is not scoped itself
+
+                const bool UNSCOPED    = !h.name.contains(":");
+                const auto HANDLERNAME = !h.name.empty() && h.name.at(0) == ':' ? h.name.substr(1) : h.name;
+
+                if (!h.options.allowFlags && !UNSCOPED) {
+                    size_t colon = 0;
+                    size_t idx   = 0;
+                    size_t depth = 0;
+
+                    while ((colon = HANDLERNAME.find(':', idx)) != std::string::npos && impl->categories.size() > depth) {
+                        auto actual = HANDLERNAME.substr(idx, colon - idx);
+
+                        if (actual != impl->categories[depth])
+                            break;
+
+                        idx = colon + 1;
+                        ++depth;
+                    }
+
+                    if (depth != impl->categories.size() || HANDLERNAME.substr(idx) != LHS)
+                        continue;
+                }
+
+                if (UNSCOPED && HANDLERNAME != LHS && !h.options.allowFlags)
+                    continue;
+
+                if (h.options.allowFlags && (!LHS.starts_with(HANDLERNAME) || LHS.contains(':') /* avoid cases where a category is called the same as a handler */))
+                    continue;
+
+                ret   = h.func(LHS.c_str(), RHS.c_str());
+                found = true;
+            }
+        }
 
         if (ret.error)
             return ret;
@@ -836,9 +868,12 @@ CParseResult CConfig::parseLine(std::string line, bool dynamic) {
                 return result;
             }
 
-            impl->currentSpecialKey      = "";
-            impl->currentSpecialCategory = nullptr;
             impl->categories.pop_back();
+
+            if (impl->categories.empty()) {
+                impl->currentSpecialKey      = "";
+                impl->currentSpecialCategory = nullptr;
+            }
         } else {
             // open a category.
             if (!line.ends_with("{")) {
@@ -995,11 +1030,15 @@ CParseResult CConfig::parseFile(const char* file) {
 }
 
 CParseResult CConfig::parseDynamic(const char* line) {
-    return parseLine(line, true);
+    auto ret = parseLine(line, true);
+    impl->currentSpecialCategory = nullptr;
+    return ret;
 }
 
 CParseResult CConfig::parseDynamic(const char* command, const char* value) {
-    return parseLine(std::string{command} + "=" + std::string{value}, true);
+    auto ret = parseLine(std::string{command} + "=" + std::string{value}, true);
+    impl->currentSpecialCategory = nullptr;
+    return ret;
 }
 
 void CConfig::clearState() {
